# © 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
import logging

from odoo import models, fields, api
from odoo.exceptions import UserError, ValidationError

_logger = logging.getLogger(__name__)
# HELP : https://chemistry.apache.org/python/docs/examples.html


class ScopInstance(models.Model):
    _inherit = ["scop.instance", 'cgscop.alfresco']

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    is_type_membership = fields.Boolean("Adhésions")
    date_membership = fields.Date(
        string="Date d'adhésion statistique",
    )
    instance_partner_ids = fields.One2many(
        comodel_name='scop.instance.partner',
        inverse_name='instance_id',
        string='Coopératives',
        required=False)
    partners_count = fields.Integer(
        string='Nombre de lignes de commandes',
        compute='_compute_partners_count')
    comments = fields.Text('Commentaires')

    email_count = fields.Integer(
        string='Nombre de mails',
        compute='_compute_emails'
    )

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    @api.multi
    def _compute_partners_count(self):
        for r in self:
            r.partners_count = len(r.instance_partner_ids)

    @api.multi
    def _compute_emails(self):
        for r in self:
            partners = r._get_partners()
            email_ids = self.env['mail.mail'].search([
                ('mail_message_id', 'in', partners.mapped('message_ids').ids),
                ('subtype_id', '=', self.env.ref(
                    'cgscop_adhesion_alfodoo.'
                    'mail_message_subtype_adhesion').id)
            ],)
            r.email_count = len(email_ids)

    # ------------------------------------------------------
    # Onchange fields
    # ------------------------------------------------------
    @api.onchange('date')
    def onchange_date(self):
        if not self.date_membership:
            self.date_membership = fields.Date.to_date(self.date)

    # ------------------------------------------------------
    # Business method
    # ------------------------------------------------------
    @api.multi
    def get_report_datas(self):
        """
        :return: dict of stats (group by ur and cooperative form)
        - datas_per_ur_type : list of dict per ur and type
        - datas_stats_ur : list of dict per ur
        - datas_stats_global : dict of stats
        """
        partners = self._get_partners()

        # We use LM instead of res_partner to get more info ...
        # ... fields of res_partner are on LM (delegate = true)
        lm_ids = list()
        for partner in partners:
            lm_partner = partner.get_lm_adhesion()
            lm_ids.append(lm_partner.id)

        partners_lm = self.env['liste.ministere'].browse(lm_ids)

        ur_global = partners_lm.read_group(
            domain=[('id', 'in', partners_lm.ids)],
            fields=['ur_id'],
            groupby=['ur_id', 'eff_tt', 'eff_sal_assoc'],
            orderby='ur_id',
            lazy=True
        )
        datas_stats_global = {
            'ur_global': ur_global,
            'partners': partners_lm,
        }
        return datas_stats_global

    def _get_partners(self):
        """
        map o2m on partner_id
        """
        return self.instance_partner_ids.mapped('partner_id')

    @api.multi
    def create_report_adhesion(self, partners):
        """
        Generic function to create report adhésion CG
        :return:
        """
        return self.env.ref('cgscop_adhesion.cgscop_adhesion_report'). \
            report_action(partners)

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def action_add_coop(self):
        """
        Display wizard to attach coop to instance
        :return:
        """
        self.ensure_one()
        wizard = self.env['scop.instance.wizard'].\
            create({'instance_id': self.id})
        return {
                'name': "Ajouter des coops",
                'type': 'ir.actions.act_window',
                'view_mode': 'form',
                'res_model': 'scop.instance.wizard',
                'res_id': wizard.id,
                'target': 'new'
                }

    @api.multi
    def create_report_instance_adhesion(self):
        """
        Create report for "états préparatoires"
        :return:
        """
        return self.env.ref(
            'cgscop_instance.cgscop_instance_adhesion_report'). \
            report_action(self)

    @api.multi
    def create_report_adhesion_all(self):
        """
        Create report adhésion CG ALL
        :return:
        """
        partners = self._get_partners().filtered(
            lambda p: p.project_status == '6_suivi')
        if len(partners) < 1:
            raise UserError(
                'Aucune coopérative n\'est en statut "suivi"')
        return self.create_report_adhesion(partners)

    @api.multi
    def create_report_adhesion_per_ur(self):
        """
        Create report adhésion CG PER UR
        :return:
        """
        partners = self._get_partners().filtered(
            lambda i: i.project_status == '6_suivi')
        if len(partners) < 1:
            raise UserError(
                'Aucune coopérative n\'est en statut "suivi"')
        urs = partners.mapped('ur_id')
        wizard = self.env['scop.instance.report.wizard'].create({
            'partner_ids': [(6, 0, partners.ids)],
            'ur_ids': [(6, 0, urs.ids)],
        })

        return {
            'name': 'Courriers d\'adhésion par UR',
            'type': 'ir.actions.act_window',
            'view_mode': 'form',
            'res_model': 'scop.instance.report.wizard',
            'res_id': wizard.id,
            'target': 'new',
            'flags': {'mode': 'readonly'}
        }

    @api.multi
    def action_send_mail_adhesion(self):
        """
        Open wizard to send mail adhésions
        """
        partners = self._get_partners().filtered(
            lambda i: i.project_status == '6_suivi')
        if len(partners) < 1:
            raise UserError(
                'Aucune coopérative n\'est en statut "suivi"')
        wizard = self.env['scop.instance.mail.wizard'].create({})

        return {
            'name': 'Envoyer les mails d\'adhésion',
            'type': 'ir.actions.act_window',
            'view_mode': 'form',
            'res_model': 'scop.instance.mail.wizard',
            'res_id': wizard.id,
            'target': 'new',
            'domain': [('partner_ids', 'in', partners.ids)],
        }

    def action_show_coop(self):
        """
        Display tree view of coop linked to instance
        :return:
        """
        self.ensure_one()
        partner_ids = self._get_partners().ids
        tree_view = self.env.ref(
            'cgscop_instance.res_partner_instance_coops_tree').id
        return {
            'type': 'ir.actions.act_window',
            'name': 'Coopératives',
            'views': [
                [tree_view, "tree"],
                [False, "form"]
            ],
            'view_mode': 'tree',
            'res_model': 'res.partner',
            'target': 'current',
            'domain': [["id", "in", partner_ids]],
        }

    def action_show_emails(self):
        """
        Open email related to partners in instance and subtype adhésion
        """
        partners = self._get_partners()
        return {
            'name': 'Etat des mails envoyés',
            'type': 'ir.actions.act_window',
            'view_mode': 'form',
            'views': [(False, 'tree'), (False, 'form')],
            'res_model': 'mail.mail',
            'domain': [
                ('mail_message_id', 'in', partners.mapped('message_ids').ids),
                ('subtype_id', '=', self.env.ref(
                    'cgscop_adhesion_alfodoo.'
                    'mail_message_subtype_adhesion').id)
            ],
        }

    # ------------------------------------------------------
    # Validation
    # ------------------------------------------------------
    def scop_valid_cg_mass(self):
        """
        Select all coop in state positive & status "soumis cg" to be validated
        - use scop_valid_cg
        - set lf_adhesion qualified
        - affect new start_stat if needed
        - log success or error
        - open wizard to send mail to all validated coops
        """
        partners_to_validate = self.env.context.get(
            'partners_to_validate', False)
        if not partners_to_validate:
            partners_to_validate = self.instance_partner_ids.filtered(
                lambda p: p.state == 'positive' and p.project_status == '5_cg')
        validated_partners = list()
        for r in partners_to_validate:
            partner = r.partner_id
            try:
                # scop_valid_cg
                partner.with_context(
                    origin_instance_id=self.id).scop_valid_cg()
                # set membership stat period
                membership_period = self.env['scop.membership.period'].search(
                    [('partner_id', '=', partner.id),
                     ('start', '=', self.date),
                     ('end', '=', False)], limit=1)
                if membership_period.start_stat != r.statistic_membership_date:
                    membership_period.update({
                        'start_stat': r.statistic_membership_date
                    })
                # log success
                validated_partners.append(partner.id)
                r.is_success = True
                self.env.cr.commit()
            except Exception as e:
                logger = "Erreur lors la validation du dossier %s : " \
                         "%s" % (partner.name, e.args[0])
                _logger.warning(logger)
                self.env.cr.rollback()
                r.is_success = False
                r.message_error = fields.datetime.now().strftime(
                    "%d/%m/%Y - %-H:%M") + ' - Erreur : ' + e.args[0]
                self.env.cr.commit()
                if len(partners_to_validate) == 1:
                    raise ValidationError(logger)

        if len(validated_partners) > 0:
            return self.send_mass_mail_adhesion(
                self.env['res.partner'].browse(validated_partners))

    @api.multi
    def send_mass_mail_adhesion(self, partners):
        """
        Open wizard to send mail adhésion to partners of instance
        """
        self.ensure_one()
        template_id = self.env.ref(
            'cgscop_adhesion_alfodoo.email_template_adhesion')
        ir_model_data = self.env['ir.model.data']
        try:
            compose_form_id = ir_model_data.get_object_reference(
                'mail', 'email_compose_message_wizard_form')[1]
        except ValueError:
            compose_form_id = False
        ctx = {
            'default_model': 'res.partner',
            'default_res_id': partners.ids[0],
            'default_use_template': True,
            'default_template_id': template_id.id,
            'default_subtype_id': self.env.ref(
                'cgscop_adhesion_alfodoo.mail_message_subtype_adhesion').id,
            'default_composition_mode': 'mass_mail' if len(
                partners.ids) > 1 else 'comment',
            'active_ids': partners.ids
        }
        return {
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'mail.compose.message',
            'views': [(compose_form_id, 'form')],
            'view_id': compose_form_id,
            'target': 'new',
            'context': ctx,
        }

    def update_membership_date(self):
        for partner in self.instance_partner_ids:
            partner.update({
                'statistic_membership_date': self.date_membership
            })
