# © 2022 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import models, fields, api


class ScopInstancePartner(models.Model):
    _name = "scop.instance.partner"
    _description = "Statut coop par instance"

    # ------------------------------------------------------
    # Default
    # ------------------------------------------------------
    @api.model
    def _get_default_date_stat(self):
        if self.env.context.get('active_model') == 'scop.instance':
            return self.env['scop.instance'].browse(
                self.env.context['active_id']).date.date()

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    instance_id = fields.Many2one(
        comodel_name='scop.instance',
        string='Instance',
        required=True)

    partner_id = fields.Many2one(
        comodel_name='res.partner',
        string='Coopératives',
        required=True)
    display_name = fields.Char(
        string='Coopérative',
        related='partner_id.display_name')
    city = fields.Char(
        string="Ville",
        related='partner_id.city')
    ur_id = fields.Many2one(
        comodel_name='union.regionale',
        string='UR',
        related='partner_id.ur_id')
    phone = fields.Char(
        string="Téléphone",
        related='partner_id.phone')
    email = fields.Char(
        string='Email',
        related='partner_id.email')
    project_status = fields.Selection(
        string='Statut projet',
        related='partner_id.project_status')

    state = fields.Selection(
        string='Avis',
        selection=[('positive', 'Favorable'),
                   ('negative', 'Défavorable')],
        default='positive',
        required=True, track_visibility='onchange')
    amount_adhesion = fields.Float(
        string='Montant Adhésion',
        compute='_compute_amount_adhesion', store=True,
        track_visibility='onchange')
    comments = fields.Text('Commentaires', track_visibility='onchange')
    statistic_membership_date = fields.Date(
        string='Date d\'adhésion statistique',
        default=lambda self: self._get_default_date_stat())

    is_success = fields.Boolean('Succès', default=True)
    message_error = fields.Text('Message d\'erreur')

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------
    """
    Contrainte : Unicité clé partner_id-instance_id
    """
    _sql_constraints = [
        ('uniqueness_partner_instance_key',
         'unique(partner_id, instance_id)',
         "Une coop ne peut être présente qu'une seule fois par instance"),
    ]

    # ------------------------------------------------------
    # Compute
    # ------------------------------------------------------
    @api.multi
    @api.depends('partner_id', 'state')
    def _compute_amount_adhesion(self):
        for r in self:
            if r.state == 'positive':
                lm = r.partner_id.lm_adhesion_id \
                    if r.partner_id.lm_adhesion_id \
                    else r.partner_id.get_lm_adhesion(r.instance_id.date.year)
                effectif = lm.eff_tt
                r.amount_adhesion = \
                    r.partner_id.get_theorical_amount_adhesion(effectif)
            else:
                r.amount_adhesion = 0

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    @api.multi
    def switch_state(self):
        self.ensure_one()
        if self.state == 'positive':
            self.state = 'negative'
        else:
            self.state = 'positive'

    @api.multi
    def action_free_coop_from_instance(self):
        """
        Free coop from instance from "scop_instance_partner" view
        :return:
        """
        self.ensure_one()
        # On détache la coop de l'instance
        self.partner_id.write({
            'instance_ids': [(3, self.id)]}
        )
        # On supprime l'objet "statut coop par instance" associé
        self.unlink()

    def try_push_again(self):
        message = (
                "<p>" + self.message_error + ".</p>" +
                "<p><strong>Voulez-vous tenter de valider à "
                "nouveau cette coop ?</strong></p>"
        )
        action = 'instance_id.with_context(partners_to_validate=' \
                 'self.env[\'scop.instance.partner\'].' \
                 'browse(' + str(self.id) + ')).scop_valid_cg_mass()'
        message_id = self.env['message.wizard'].create({
            'message': message,
            'cancel_button': True,
            'action': action,
        })

        return {
            'name': 'Erreur de validation pour ' + self.partner_id.name,
            'type': 'ir.actions.act_window',
            'view_mode': 'form',
            'res_model': 'message.wizard',
            'res_id': message_id.id,
            'target': 'new'
        }
